@@ -7,6 +7,7 @@ import json
 import pickle
 import warnings
 from pathlib import Path
+from typing import Any, List, Optional
 
 import numpy as np
 import pandas as pd
@@ -14,15 +15,16 @@ import pandas as pd
 from responsibleai._input_processing import _convert_to_list
 from responsibleai._interfaces import Dataset, RAIInsightsData
 from responsibleai._internal.constants import ManagerNames, Metadata, SKLearn
-from responsibleai._managers.causal_manager import CausalManager
-from responsibleai._managers.counterfactual_manager import \
-    CounterfactualManager
-from responsibleai._managers.error_analysis_manager import ErrorAnalysisManager
-from responsibleai._managers.explainer_manager import ExplainerManager
 from responsibleai.exceptions import UserConfigValidationException
+from responsibleai.managers.causal_manager import CausalManager
+from responsibleai.managers.counterfactual_manager import CounterfactualManager
+from responsibleai.managers.error_analysis_manager import ErrorAnalysisManager
+from responsibleai.managers.explainer_manager import ExplainerManager
 from responsibleai.rai_insights.constants import ModelTask
+from responsibleai.utils import _is_classifier
 
 _DATA = 'data'
+_PREDICTIONS = 'predictions'
 _DTYPES = 'dtypes'
 _TRAIN = 'train'
 _TEST = 'test'
@@ -37,47 +39,24 @@ _CATEGORICAL_FEATURES = 'categorical_features'
 _META_JSON = Metadata.META_JSON
 _TRAIN_LABELS = 'train_labels'
 _JSON_EXTENSION = '.json'
+_PREDICT = 'predict'
+_PREDICT_PROBA = 'predict_proba'
 
 
 class RAIInsights(object):
-
     """Defines the top-level Model Analysis API.
     Use RAIInsights to analyze errors, explain the most important
     features, compute counterfactuals and run causal analysis in a
     single API.
-    :param model: The model to compute RAI insights for.
-        A model that implements sklearn.predict or sklearn.predict_proba
-        or function that accepts a 2d ndarray.
-    :type model: object
-    :param train: The training dataset including the label column.
-    :type train: pandas.DataFrame
-    :param test: The test dataset including the label column.
-    :type test: pandas.DataFrame
-    :param target_column: The name of the label column.
-    :type target_column: str
-    :param task_type: The task to run, can be `classification` or
-        `regression`.
-    :type task_type: str
-    :param categorical_features: The categorical feature names.
-    :type categorical_features: list[str]
-    :param classes: The class labels in the training dataset
-    :type classes: ndarray
-    :param serializer: Picklable custom serializer with save and load
-        methods for custom model serialization.
-        The save method writes the model to file given a parent directory.
-        The load method returns the deserialized model from the same
-        parent directory.
-    :type serializer: object
     """
 
-    def __init__(self, model, train, test, target_column,
-                 task_type, categorical_features=None, classes=None,
-                 serializer=None,
+    def __init__(self, model: Any, train: pd.DataFrame, test: pd.DataFrame,
+                 target_column: str, task_type: str,
+                 categorical_features: Optional[List[str]] = None,
+                 classes: Optional[np.ndarray] = None,
+                 serializer: Optional[Any] = None,
                  maximum_rows_for_test: int = 5000):
-        """Defines the top-level Model Analysis API.
-        Use RAIInsights to analyze errors, explain the most important
-        features, compute counterfactuals and run causal analysis in a
-        single API.
+        """Creates an RAIInsights object.
         :param model: The model to compute RAI insights for.
             A model that implements sklearn.predict or sklearn.predict_proba
             or function that accepts a 2d ndarray.
@@ -94,16 +73,18 @@ class RAIInsights(object):
         :param categorical_features: The categorical feature names.
         :type categorical_features: list[str]
         :param classes: The class labels in the training dataset
-        :type classes: ndarray
+        :type classes: numpy.ndarray
         :param serializer: Picklable custom serializer with save and load
-            methods defined for model that is not serializable. The save
-            method returns a dictionary state and load method returns the
-            model.
+            methods for custom model serialization.
+            The save method writes the model to file given a parent directory.
+            The load method returns the deserialized model from the same
+            parent directory.
         :type serializer: object
         :param maximum_rows_for_test: Limit on size of test data
             (for performance reasons)
         :type maximum_rows_for_test: int
         """
+        categorical_features = categorical_features or []
         self._validate_model_analysis_input_parameters(
             model=model, train=train, test=test,
             target_column=target_column, task_type=task_type,
@@ -163,12 +144,12 @@ class RAIInsights(object):
             return None
 
     def _validate_model_analysis_input_parameters(
-            self, model, train, test, target_column,
-            task_type, categorical_features, classes,
+            self, model: Any, train: pd.DataFrame, test: pd.DataFrame,
+            target_column: str, task_type: str,
+            categorical_features: List[str], classes: np.ndarray,
             serializer,
             maximum_rows_for_test: int):
-        """
-        Validate the inputs for RAIInsights class.
+        """Validate the inputs for the RAIInsights constructor.
 
         :param model: The model to compute RAI insights for.
             A model that implements sklearn.predict or sklearn.predict_proba
@@ -186,7 +167,7 @@ class RAIInsights(object):
         :param categorical_features: The categorical feature names.
         :type categorical_features: list[str]
         :param classes: The class labels in the training dataset
-        :type classes: ndarray
+        :type classes: numpy.ndarray
         :param serializer: Picklable custom serializer with save and load
             methods defined for model that is not serializable. The save
             method returns a dictionary state and load method returns the
@@ -266,6 +247,23 @@ class RAIInsights(object):
                                f"{list(difference_set)}")
                     raise UserConfigValidationException(message)
 
+                for column in categorical_features:
+                    try:
+                        np.unique(train[column])
+                    except Exception:
+                        raise UserConfigValidationException(
+                            "Error finding unique values in column {0}. "
+                            "Please check your train data.".format(column)
+                        )
+
+                    try:
+                        np.unique(test[column])
+                    except Exception:
+                        raise UserConfigValidationException(
+                            "Error finding unique values in column {0}. "
+                            "Please check your test data.".format(column)
+                        )
+
             if classes is not None and task_type == \
                     ModelTask.CLASSIFICATION:
                 if len(set(train[target_column].unique()) -
@@ -291,6 +289,8 @@ class RAIInsights(object):
                 small_test_data = test.iloc[0:1].drop(
                     [target_column], axis=1)
 
+                small_train_features_before = list(small_train_data.columns)
+
                 # Run predict() of the model
                 try:
                     model.predict(small_train_data)
@@ -300,6 +300,9 @@ class RAIInsights(object):
                         'The model passed cannot be used for'
                         ' getting predictions via predict()'
                     )
+                self._validate_features_same(small_train_features_before,
+                                             small_train_data,
+                                             SKLearn.PREDICT)
 
                 # Run predict_proba() of the model
                 if task_type == ModelTask.CLASSIFICATION:
@@ -311,9 +314,12 @@ class RAIInsights(object):
                             'The model passed cannot be used for'
                             ' getting predictions via predict_proba()'
                         )
+                self._validate_features_same(small_train_features_before,
+                                             small_train_data,
+                                             SKLearn.PREDICT_PROBA)
 
                 if task_type == ModelTask.REGRESSION:
-                    if hasattr(model, 'predict_proba'):
+                    if hasattr(model, SKLearn.PREDICT_PROBA):
                         warnings.warn(
                             'INVALID-TASK-TYPE-WARNING: The regression model'
                             'provided has a predict_proba function. '
@@ -321,7 +327,28 @@ class RAIInsights(object):
         else:
             raise UserConfigValidationException(
                 "Unsupported data type for either train or test. "
-                "Expecting pandas Dataframe for train and test."
+                "Expecting pandas DataFrame for train and test."
+            )
+
+    def _validate_features_same(self, small_train_features_before,
+                                small_train_data, function):
+        """
+        Validate the features are unmodified on the DataFrame.
+
+        :param small_train_features_before: The features saved before
+            an operation was performed.
+        :type small_train_features_before: list[str]
+        :param small_train_data: The DataFrame after the operation.
+        :type small_train_data: pandas.DataFrame
+        :param function: The name of the operation performed.
+        :type function: str
+        """
+        small_train_features_after = list(small_train_data.columns)
+        if small_train_features_before != small_train_features_after:
+            raise UserConfigValidationException(
+                ('Calling model {} function modifies '
+                 'input dataset features. Please check if '
+                 'predict function is defined correctly.').format(function)
             )
 
     @property
@@ -469,10 +496,7 @@ class RAIInsights(object):
                                  " from local explanations dimension")
             dashboard_dataset.feature_names = features
         dashboard_dataset.target_column = self.target_column
-        if (self.model is not None and
-                hasattr(self.model, SKLearn.PREDICT_PROBA) and
-                self.model.predict_proba is not None and
-                dataset is not None):
+        if _is_classifier(self.model) and dataset is not None:
             try:
                 probability_y = self.model.predict_proba(dataset)
             except Exception as ex:
@@ -497,28 +521,64 @@ class RAIInsights(object):
         with open(file_path, 'w') as file:
             file.write(content)
 
-    def save(self, path):
-        """Save the RAIInsights to the given path.
+    def _save_predictions(self, path):
+        """Save the predict() and predict_proba() output.
+
+        :param path: The directory path to save the RAIInsights to.
+        :type path: str
+        """
+        prediction_output_path = Path(path) / _PREDICTIONS
+        prediction_output_path.mkdir(parents=True, exist_ok=True)
+
+        if self.model is None:
+            return
+
+        test_without_target_column = self.test.drop(
+            [self.target_column], axis=1)
+
+        predict_output = self.model.predict(test_without_target_column)
+        self._write_to_file(
+            prediction_output_path / (_PREDICT + _JSON_EXTENSION),
+            json.dumps(predict_output.tolist()))
+
+        if hasattr(self.model, SKLearn.PREDICT_PROBA):
+            predict_proba_output = self.model.predict_proba(
+                test_without_target_column)
+            self._write_to_file(
+                prediction_output_path / (_PREDICT_PROBA + _JSON_EXTENSION),
+                json.dumps(predict_proba_output.tolist()))
+
+    def _save_data(self, path):
+        """Save the copy of raw data (train and test sets) and
+           their related metadata.
+
+        :param path: The directory path to save the RAIInsights to.
+        :type path: str
+        """
+        data_directory = Path(path) / _DATA
+        data_directory.mkdir(parents=True, exist_ok=True)
+        dtypes = self.train.dtypes.astype(str).to_dict()
+        self._write_to_file(data_directory /
+                            (_TRAIN + _DTYPES + _JSON_EXTENSION),
+                            json.dumps(dtypes))
+        self._write_to_file(data_directory / (_TRAIN + _JSON_EXTENSION),
+                            self.train.to_json(orient='split'))
+
+        dtypes = self.test.dtypes.astype(str).to_dict()
+        self._write_to_file(data_directory /
+                            (_TEST + _DTYPES + _JSON_EXTENSION),
+                            json.dumps(dtypes))
+        self._write_to_file(data_directory / (_TEST + _JSON_EXTENSION),
+                            self.test.to_json(orient='split'))
+
+    def _save_metadata(self, path):
+        """Save the metadata like target column, categorical features,
+           task type and the classes (if any).
+
         :param path: The directory path to save the RAIInsights to.
         :type path: str
         """
         top_dir = Path(path)
-        # save each of the individual managers
-        for manager in self._managers:
-            manager._save(top_dir / manager.name)
-        # save current state
-        data_directory = Path(path) / _DATA
-        data_directory.mkdir(parents=True, exist_ok=True)
-        dtypes = self.train.dtypes.astype(str).to_dict()
-        self._write_to_file(data_directory / (_TRAIN + _DTYPES),
-                            json.dumps(dtypes))
-        self._write_to_file(data_directory / (_TRAIN + _JSON_EXTENSION),
-                            self.train.to_json())
-        dtypes = self.test.dtypes.astype(str).to_dict()
-        self._write_to_file(data_directory / (_TEST + _DTYPES),
-                            json.dumps(dtypes))
-        self._write_to_file(data_directory / (_TEST + _JSON_EXTENSION),
-                            self.test.to_json())
         classes = _convert_to_list(self._classes)
         meta = {
             _TARGET_COLUMN: self.target_column,
@@ -528,6 +588,14 @@ class RAIInsights(object):
         }
         with open(top_dir / _META_JSON, 'w') as file:
             json.dump(meta, file)
+
+    def _save_model(self, path):
+        """Save the model and the serializer (if any).
+
+        :param path: The directory path to save the RAIInsights to.
+        :type path: str
+        """
+        top_dir = Path(path)
         if self._serializer is not None:
             # save the model
             self._serializer.save(self.model, top_dir)
@@ -545,30 +613,62 @@ class RAIInsights(object):
             with open(top_dir / _MODEL_PKL, 'wb') as file:
                 pickle.dump(self.model, file)
 
-    @staticmethod
-    def load(path):
-        """Load the RAIInsights from the given path.
-        :param path: The directory path to load the RAIInsights from.
+    def _save_managers(self, path):
+        """Save the state of individual managers.
+
+        :param path: The directory path to save the RAIInsights to.
         :type path: str
-        :return: The RAIInsights object after loading.
-        :rtype: RAIInsights
         """
-        # create the RAIInsights without any properties using the __new__
-        # function, similar to pickle
-        inst = RAIInsights.__new__(RAIInsights)
         top_dir = Path(path)
-        # load current state
+        # save each of the individual managers
+        for manager in self._managers:
+            manager._save(top_dir / manager.name)
+
+    def save(self, path):
+        """Save the RAIInsights to the given path.
+
+        :param path: The directory path to save the RAIInsights to.
+        :type path: str
+        """
+        self._save_managers(path)
+        self._save_data(path)
+        self._save_metadata(path)
+        self._save_model(path)
+        self._save_predictions(path)
+
+    @staticmethod
+    def _load_data(inst, path):
+        """Load the raw data (train and test sets).
+
+        :param inst: RAIInsights object instance.
+        :type inst: RAIInsights
+        :param path: The directory path to data location.
+        :type path: str
+        """
         data_directory = Path(path) / _DATA
-        with open(data_directory / (_TRAIN + _DTYPES), 'r') as file:
+        with open(data_directory /
+                  (_TRAIN + _DTYPES + _JSON_EXTENSION), 'r') as file:
             types = json.load(file)
         with open(data_directory / (_TRAIN + _JSON_EXTENSION), 'r') as file:
-            train = pd.read_json(file, dtype=types)
+            train = pd.read_json(file, dtype=types, orient='split')
         inst.__dict__[_TRAIN] = train
-        with open(data_directory / (_TEST + _DTYPES), 'r') as file:
+        with open(data_directory /
+                  (_TEST + _DTYPES + _JSON_EXTENSION), 'r') as file:
             types = json.load(file)
         with open(data_directory / (_TEST + _JSON_EXTENSION), 'r') as file:
-            test = pd.read_json(file, dtype=types)
+            test = pd.read_json(file, dtype=types, orient='split')
         inst.__dict__[_TEST] = test
+
+    @staticmethod
+    def _load_metadata(inst, path):
+        """Load the metadata.
+
+        :param inst: RAIInsights object instance.
+        :type inst: RAIInsights
+        :param path: The directory path to metadata location.
+        :type path: str
+        """
+        top_dir = Path(path)
         with open(top_dir / _META_JSON, 'r') as meta_file:
             meta = meta_file.read()
         meta = json.loads(meta)
@@ -583,11 +683,21 @@ class RAIInsights(object):
 
         inst.__dict__['_' + _CLASSES] = RAIInsights._get_classes(
             task_type=meta[_TASK_TYPE],
-            train=train,
+            train=inst.__dict__[_TRAIN],
             target_column=meta[_TARGET_COLUMN],
             classes=classes
         )
 
+    @staticmethod
+    def _load_model(inst, path):
+        """Load the model.
+
+        :param inst: RAIInsights object instance.
+        :type inst: RAIInsights
+        :param path: The directory path to model location.
+        :type path: str
+        """
+        top_dir = Path(path)
         serializer_path = top_dir / _SERIALIZER
         if serializer_path.exists():
             with open(serializer_path, 'rb') as file:
@@ -596,9 +706,26 @@ class RAIInsights(object):
             inst.__dict__[_MODEL] = serializer.load(top_dir)
         else:
             inst.__dict__['_' + _SERIALIZER] = None
-            with open(top_dir / _MODEL_PKL, 'rb') as file:
-                inst.__dict__[_MODEL] = pickle.load(file)
+            try:
+                with open(top_dir / _MODEL_PKL, 'rb') as file:
+                    inst.__dict__[_MODEL] = pickle.load(file)
+            except Exception:
+                warnings.warn(
+                    'ERROR-LOADING-USER-MODEL: '
+                    'There was an error loading the user model. '
+                    'Some of RAI dashboard features may not work.')
+                inst.__dict__[_MODEL] = None
 
+    @staticmethod
+    def _load_managers(inst, path):
+        """Load the model.
+
+        :param inst: RAIInsights object instance.
+        :type inst: RAIInsights
+        :param path: The directory path to manager location.
+        :type path: str
+        """
+        top_dir = Path(path)
         # load each of the individual managers
         manager_map = {
             ManagerNames.CAUSAL: CausalManager,
@@ -615,4 +742,24 @@ class RAIInsights(object):
             managers.append(manager)
 
         inst.__dict__['_' + _MANAGERS] = managers
+
+    @staticmethod
+    def load(path):
+        """Load the RAIInsights from the given path.
+
+        :param path: The directory path to load the RAIInsights from.
+        :type path: str
+        :return: The RAIInsights object after loading.
+        :rtype: RAIInsights
+        """
+        # create the RAIInsights without any properties using the __new__
+        # function, similar to pickle
+        inst = RAIInsights.__new__(RAIInsights)
+
+        # load current state
+        RAIInsights._load_data(inst, path)
+        RAIInsights._load_metadata(inst, path)
+        RAIInsights._load_model(inst, path)
+        RAIInsights._load_managers(inst, path)
+
         return inst
