@@ -52,7 +52,7 @@ class TestModelAnalysisValidations:
                 target_column='bad_target',
                 task_type='classification',
                 maximum_rows_for_test=len(y_test) - 1)
-        assert "The test data has 30 rows, but limit is set to 29 rows" in \
+        assert "The test data has 31 rows, but limit is set to 30 rows" in \
             str(ucve.value)
         assert "Please resample the test data or " +\
             "adjust maximum_rows_for_test" in \
@@ -279,7 +279,7 @@ class TestModelAnalysisValidations:
                 target_column='target',
                 task_type='classification')
         assert "Unsupported data type for either train or test. " + \
-            "Expecting pandas Dataframe for train and test." in str(ucve.value)
+            "Expecting pandas DataFrame for train and test." in str(ucve.value)
 
     def test_train_labels(self):
         X_train, X_test, y_train, y_test, _, _ = \
