@@ -1,16 +1,17 @@
 # Copyright (c) Microsoft Corporation
 # Licensed under the MIT License.
 
+import os
+import pickle
 from pathlib import Path
 from tempfile import TemporaryDirectory
-
-import numpy as np
-import os
-import pandas as pd
-import pytest
 from uuid import UUID
 
-from responsibleai import RAIInsights, ModelTask
+import numpy as np
+import pandas as pd
+import pytest
+
+from responsibleai import ModelTask, RAIInsights
 from responsibleai._internal.constants import ManagerNames
 from responsibleai._tools.shared.state_directory_management import \
     DirectoryManager
@@ -18,9 +19,10 @@ from responsibleai._tools.shared.state_directory_management import \
 from .causal_manager_validator import validate_causal
 from .common_utils import (create_adult_income_dataset,
                            create_binary_classification_dataset,
-                           create_boston_data, create_cancer_data,
+                           create_cancer_data,
                            create_complex_classification_pipeline,
-                           create_iris_data, create_models_classification,
+                           create_housing_data, create_iris_data,
+                           create_models_classification,
                            create_models_regression)
 from .counterfactual_manager_validator import validate_counterfactual
 from .error_analysis_validator import (setup_error_analysis,
@@ -146,10 +148,11 @@ class TestRAIInsights(object):
 
     @pytest.mark.parametrize('manager_type', [ManagerNames.CAUSAL,
                                               ManagerNames.COUNTERFACTUAL,
-                                              ManagerNames.EXPLAINER])
-    def test_modelanalysis_boston(self, manager_type):
+                                              ManagerNames.EXPLAINER,
+                                              ManagerNames.ERROR_ANALYSIS])
+    def test_rai_insights_housing(self, manager_type):
         X_train, X_test, y_train, y_test, feature_names = \
-            create_boston_data()
+            create_housing_data()
         X_train = pd.DataFrame(X_train, columns=feature_names)
         X_test = pd.DataFrame(X_test, columns=feature_names)
         models = create_models_regression(X_train, y_train)
@@ -157,112 +160,16 @@ class TestRAIInsights(object):
         X_test[LABELS] = y_test
 
         manager_args = {
-            ManagerParams.DESIRED_RANGE: [10, 20],
-            ManagerParams.TREATMENT_FEATURES: ['CHAS'],
+            ManagerParams.DESIRED_RANGE: [5, 10],
+            ManagerParams.TREATMENT_FEATURES: ['AveRooms'],
             ManagerParams.MAX_CAT_EXPANSION: 12,
             ManagerParams.FEATURE_IMPORTANCE: True
         }
 
         for model in models:
-            run_rai_insights(model, X_train, X_test, LABELS, ['CHAS'],
+            run_rai_insights(model, X_train, X_test, LABELS, [],
                              manager_type, manager_args)
 
-    def test_rai_insights_empty_save_load_save(self):
-        X_train, y_train, X_test, y_test, classes = \
-            create_binary_classification_dataset()
-
-        models = create_models_classification(X_train, y_train)
-        X_train[LABELS] = y_train
-        X_test[LABELS] = y_test
-
-        for model in models:
-            rai_insights = RAIInsights(
-                model, X_train, X_test,
-                LABELS,
-                categorical_features=None,
-                task_type=ModelTask.CLASSIFICATION)
-
-            with TemporaryDirectory() as tmpdir:
-                save_1 = Path(tmpdir) / "first_save"
-                save_2 = Path(tmpdir) / "second_save"
-
-                # Save it
-                rai_insights.save(save_1)
-
-                # Load
-                rai_2 = RAIInsights.load(save_1)
-
-                # Validate, but this isn't the main check
-                validate_rai_insights(
-                    rai_2, X_train, X_test,
-                    LABELS, ModelTask.CLASSIFICATION, None)
-
-                # Save again (this is where Issue #1046 manifested)
-                rai_2.save(save_2)
-
-    @pytest.mark.parametrize('manager_type', [ManagerNames.CAUSAL,
-                                              ManagerNames.ERROR_ANALYSIS,
-                                              ManagerNames.EXPLAINER,
-                                              ManagerNames.COUNTERFACTUAL])
-    def test_rai_insights_save_load_add_save(self, manager_type):
-        data_train, data_test, y_train, y_test, categorical_features, \
-            continuous_features, target_name, classes = \
-            create_adult_income_dataset()
-        X_train = data_train.drop([target_name], axis=1)
-
-        model = create_complex_classification_pipeline(
-            X_train, y_train, continuous_features, categorical_features)
-
-        # Cut down size for counterfactuals, in the interests of speed
-        if manager_type == ManagerNames.COUNTERFACTUAL:
-            data_test = data_test[0:1]
-
-        rai_insights = RAIInsights(
-            model, data_train, data_test,
-            target_name,
-            categorical_features=categorical_features,
-            task_type=ModelTask.CLASSIFICATION)
-
-        with TemporaryDirectory() as tmpdir:
-            save_1 = Path(tmpdir) / "first_save"
-            save_2 = Path(tmpdir) / "second_save"
-
-            # Save it
-            rai_insights.save(save_1)
-
-            # Load
-            rai_2 = RAIInsights.load(save_1)
-
-            # Call a single manager
-            if manager_type == ManagerNames.CAUSAL:
-                rai_2.causal.add(
-                    treatment_features=['age', 'hours_per_week']
-                )
-            elif manager_type == ManagerNames.COUNTERFACTUAL:
-                rai_2.counterfactual.add(
-                    total_CFs=10,
-                    desired_class='opposite',
-                    feature_importance=False
-                )
-            elif manager_type == ManagerNames.ERROR_ANALYSIS:
-                rai_2.error_analysis.add()
-            elif manager_type == ManagerNames.EXPLAINER:
-                rai_2.explainer.add()
-            else:
-                raise ValueError(
-                    "Bad manager_type: {0}".format(manager_type))
-
-            rai_2.compute()
-
-            # Validate, but this isn't the main check
-            validate_rai_insights(
-                rai_2, data_train, data_test,
-                target_name, ModelTask.CLASSIFICATION,
-                categorical_features=categorical_features)
-
-            # Save again (this is where Issue #1046 manifested)
-            rai_2.save(save_2)
-
 
 def run_rai_insights(model, train_data, test_data, target_column,
                      categorical_features, manager_type,
@@ -332,9 +239,13 @@ def run_rai_insights(model, train_data, test_data, target_column,
         # save the rai_insights
         rai_insights.save(path)
 
+        # Validate the common set of state produced when rai insights
+        # are saved on the disk.
+        validate_common_state_directories(path, task_type)
+
         # Validate the directory structure of the state saved
         # by the managers.
-        validate_state_directory(path, manager_type)
+        validate_component_state_directory(path, manager_type)
 
         # load the rai_insights
         rai_insights = RAIInsights.load(path)
@@ -355,7 +266,37 @@ def run_rai_insights(model, train_data, test_data, target_column,
             validate_explainer(rai_insights, train_data, test_data, classes)
 
 
-def validate_state_directory(path, manager_type, classes=None):
+def validate_common_state_directories(path, task_type):
+    all_other_files = os.listdir(path)
+    assert "meta.json" in all_other_files
+    assert "model.pkl" in all_other_files
+
+    model = None
+    with open(path / 'model.pkl', 'rb') as file:
+        model = pickle.load(file)
+
+    predictions_path = path / "predictions"
+    assert predictions_path.exists()
+    all_predictions_files = os.listdir(predictions_path)
+    if model is not None:
+        assert "predict.json" in all_predictions_files
+        if task_type == ModelTask.CLASSIFICATION:
+            assert "predict_proba.json" in all_predictions_files
+        else:
+            assert "predict_proba.json" not in all_predictions_files
+    else:
+        assert len(all_predictions_files) == 0
+
+    data_path = path / "data"
+    assert data_path.exists()
+    all_data_files = os.listdir(data_path)
+    assert "train.json" in all_data_files
+    assert "traindtypes.json" in all_data_files
+    assert "test.json" in all_data_files
+    assert "testdtypes.json" in all_data_files
+
+
+def validate_component_state_directory(path, manager_type):
     all_dirs = os.listdir(path)
     assert manager_type in all_dirs
     all_component_paths = os.listdir(path / manager_type)
@@ -399,7 +340,7 @@ def validate_rai_insights(
     pd.testing.assert_frame_equal(rai_insights.test, test_data)
     assert rai_insights.target_column == target_column
     assert rai_insights.task_type == task_type
-    assert rai_insights.categorical_features == categorical_features
+    assert rai_insights.categorical_features == (categorical_features or [])
     if task_type == ModelTask.CLASSIFICATION:
         classes = train_data[target_column].unique()
         classes.sort()
